@@ -77,8 +77,6 @@ void adjust(TrformStr *TrPtr, aPrefs *prefs)
 #endif
 	SetAdjustDefaults(&aP);
 
-	//PrintError ("MRDL: In adjust");
-
 	switch( prefs->mode & 7 )// Should we use prefs, or read from script?
 	{
 		case _insert:
@@ -455,8 +453,8 @@ of the image that has to be converted.*/
 void MyMakePano( TrformStr *TrPtr, aPrefs *aP, int imageNum )
 {
 	struct 	MakeParams	mp,mpinv;
-	fDesc 	stack[15], fD;		// Parameters for execute 
-	fDesc 	invstack[15], finvD;		// Invers Parameters for execute 
+	fDesc        stack[15], fD;		// Parameters for execute 
+	fDesc        invstack[15], finvD;		// Invers Parameters for execute 
 	void	*morph[3];	
 
 	int 	i,k, kstart, kend, color;
@@ -505,8 +503,8 @@ void MyMakePano( TrformStr *TrPtr, aPrefs *aP, int imageNum )
 
 		if( TrPtr->success != 0)
 		{
-			fD.func = execute_stack; fD.param = stack;
-			finvD.func = execute_stack; finvD.param = invstack;
+			fD.func = execute_stack_new; fD.param = stack;
+			finvD.func = execute_stack_new; finvD.param = invstack;
 
 			transFormEx( TrPtr,  &fD , &finvD , k, imageNum );
 		}
@@ -551,8 +549,8 @@ void ExtractStill( TrformStr *TrPtr , aPrefs *aP )
 		
 		if( TrPtr->success != 0)
 		{
-			fD.func = execute_stack; fD.param = stack;
-			fDinv.func = execute_stack; fDinv.param = stackinv;
+			fD.func = execute_stack_new; fD.param = stack;
+			fDinv.func = execute_stack_new; fDinv.param = stackinv;
 			transFormEx( TrPtr, &fD, &fDinv, k, 1 );
 		}
 	}
@@ -565,6 +563,7 @@ void SetMakeParams( struct fDesc *stack, struct MakeParams *mp, Image *im , Imag
 {
 	int 		i;
 	double		a,b;						// field of view in rad
+        double          tx,ty, tpara;               // temporary variables
 /* Joost Nieuwenhuijse, 3 feb 2005: Fix for cropping bug
    If a script containing the 'C' crop parameter was stitched by PTStitcher,
    it would fail if the cropping area is partially outside the source image.
@@ -618,38 +617,107 @@ void SetMakeParams( struct fDesc *stack, struct MakeParams *mp, Image *im , Imag
 					mp->mt,
 					0 );
 
+#if 0
+        switch (pn->format)
+        {
+        case _rectilinear:
+            mp->distance        = (double) pn->width / (2.0 * tan(b/2.0));
+            if(im->format == _rectilinear) // rectilinear image
+            {
+                mp->scale[0] = ((double)pn->hfov / im->hfov) * 
+                                        (a /(2.0 * tan(a/2.0))) * ((double)image_selection_width/(double) pn->width)
+                                        * 2.0 * tan(b/2.0) / b; 
 
-	if(pn->format == _rectilinear)									// rectilinear panorama
-	{
-		mp->distance 	= (double) pn->width / (2.0 * tan(b/2.0));
-		if(im->format == _rectilinear)										// rectilinear image
-		{
-			mp->scale[0] = ((double)pn->hfov / im->hfov) * 
-						   (a /(2.0 * tan(a/2.0))) * ((double)image_selection_width/(double) pn->width)
-						   * 2.0 * tan(b/2.0) / b; 
+            }
+            else //  pamoramic or fisheye image
+            {
+                    mp->scale[0] = ((double)pn->hfov / im->hfov) * ((double)image_selection_width/ (double) pn->width)
+                                            * 2.0 * tan(b/2.0) / b; 
+            }
+            break;
+        case _equirectangular:
+        case _fisheye_ff:
+        case _panorama:
+        case _mercator:
+        case _sinusoidal:
+            // horizontal pixels per degree
+            mp->distance        = ((double) pn->width) / b;
+            if(im->format == _rectilinear) // rectilinear image
+            {
+                    mp->scale[0] = ((double)pn->hfov / im->hfov) * (a /(2.0 * tan(a/2.0))) * ((double)image_selection_width)/ ((double) pn->width); 
+            }
+            else //  pamoramic or fisheye image
+            {
+                    mp->scale[0] = ((double)pn->hfov / im->hfov) * ((double)image_selection_width)/ ((double) pn->width); 
+            }
+            break;
+        case _stereographic:
+        case _trans_mercator:
+        default:
+            break;
+        }
+        mp->scale[1]    = mp->scale[0];
 
-		}
-		else 																//  pamoramic or fisheye image
-		{
-			mp->scale[0] = ((double)pn->hfov / im->hfov) * ((double)image_selection_width/ (double) pn->width)
-						   * 2.0 * tan(b/2.0) / b; 
-		}
-	}
-	else																// equirectangular or panoramic or fisheye
-	{
-		mp->distance 	= ((double) pn->width) / b;
-		if(im->format == _rectilinear)										// rectilinear image
-		{
-			mp->scale[0] = ((double)pn->hfov / im->hfov) * (a /(2.0 * tan(a/2.0))) * ((double)image_selection_width)/ ((double) pn->width); 
+        printf("\nOrig params: mp->distance: %lf, mp->scale: %lf\n\n", mp->distance, mp->scale[0]);
+#endif
 
-		}
-		else 																//  pamoramic or fisheye image
-		{
-			mp->scale[0] = ((double)pn->hfov / im->hfov) * ((double)image_selection_width)/ ((double) pn->width); 
-		}
-	}
-	mp->scale[1] 	= mp->scale[0];
+ /* Pablo d'Angelo, April 2006.
+  * Added more output projection types. Broke mp->distance and mp->scale factor calculation
+  * into separate parts, making it easier to add new projection types
+ */
+        // calculate distance
+        switch (pn->format)
+        {
+        case _rectilinear:
+            mp->distance        = (double) pn->width / (2.0 * tan(b/2.0));
+            break;
+        case _equirectangular:
+        case _fisheye_ff:
+        case _panorama:
+        case _mercator:
+        case _sinusoidal:
+            // horizontal pixels per degree
+            mp->distance        = ((double) pn->width) / b;
+            break;
+        case _stereographic:
+            tpara = 1;
+            stereographic_erect(b/2.0, 0.0, &tx, &ty, & tpara);
+            mp->distance = pn->width/(2.0*tx);
+            break;
+        case _trans_mercator:
+            tpara = 1;
+            transmercator_erect(b/2.0, 0.0, &tx, &ty, &tpara);
+            mp->distance = pn->width/(2.0*tx);
+            break;
+        default:
+            // unknown
+            PrintError ("SetMakeParams: Unsupported panorama projection");
+            // no way to report an error back to the caller...
+            mp->distance = 1;
+        }
 
+        // calculate final scaling factor, that reverses the mp->distance
+        // scaling and applies the required output scaling factor
+        switch (im->format)
+        {
+        case _rectilinear:
+            // calculate distance for this projection
+            mp->scale[0] = (double) im->width / (2.0 * tan(a/2.0)) / mp->distance;
+            break;
+        case _equirectangular:
+        case _panorama:
+        case _fisheye_ff:
+        case _fisheye_circ:
+            mp->scale[0] = ((double) im->width) / a / mp->distance;
+            break;
+        default:
+            PrintError ("SetMakeParams: Unsupported input image projection");
+            // no way to report an error back to the caller...
+            mp->scale[1] = 1;
+        }
+        mp->scale[1]    = mp->scale[0];
+
+//        printf("new params: mp->distance: %lf, mp->scale: %lf\n\n", mp->distance, mp->scale[0]);
 
 	mp->shear[0] 	= im->cP.shear_x / image_selection_height;
 	mp->shear[1] 	= im->cP.shear_y / image_selection_width;
@@ -687,6 +755,29 @@ void SetMakeParams( struct fDesc *stack, struct MakeParams *mp, Image *im , Imag
 	{
 		SetDesc(stack[i],	erect_sphere_tp,		&(mp->distance)	); i++;	// Convert panoramic to sphere
 	}
+   	else if(pn->format == _mercator)
+	{
+		SetDesc(stack[i],	erect_mercator,		&(mp->distance)	); i++;	// Convert mercator to sphere
+    }
+   	else if(pn->format == _trans_mercator)
+	{
+		SetDesc(stack[i], erect_transmercator, &(mp->distance)	); i++;	// Convert transverse mercator to sphere
+    }
+	else if(pn->format == _stereographic)
+	{
+		SetDesc(stack[i], erect_stereographic, &(mp->distance) ); i++;	// Convert stereographic to sphere
+    }
+   	else if(pn->format == _sinusoidal)
+	{
+		SetDesc(stack[i],	erect_sinusoidal,		&(mp->distance)	); i++;	// Convert sinusoidal to sphere
+	}
+   	else if(pn->format == _equirectangular) 
+	{
+		// no conversion needed		
+    } else {
+		PrintError("Projection type %d not supported, using equirectangular", pn->format);
+	}
+
 
 	SetDesc(	stack[i],	rotate_erect,		mp->rot			); i++;	// Rotate equirect. image horizontally
 	SetDesc(	stack[i],	sphere_tp_erect,	&(mp->distance)	); i++;	// Convert spherical image to equirect.
@@ -792,7 +883,7 @@ void 	SetInvMakeParams( struct fDesc *stack, struct MakeParams *mp, Image *im ,
 
 	int 		i;
 	double		a,b;							// field of view in rad
-
+    double      tx,ty,tpara;
 
 	a =	 DEG_TO_RAD( im->hfov );	// field of view in rad		
 	b =	 DEG_TO_RAD( pn->hfov );
@@ -804,7 +895,59 @@ void 	SetInvMakeParams( struct fDesc *stack, struct MakeParams *mp, Image *im ,
 				mp->mt, 
 				1 );
 
+        // dangelo: added mercator, sinusoidal and stereographic projection
+        switch (pn->format)
+        {
+        case _rectilinear:
+            mp->distance        = (double) pn->width / (2.0 * tan(b/2.0));
+            break;
+        case _equirectangular:
+        case _fisheye_ff:
+        case _panorama:
+        case _mercator:
+        case _sinusoidal:
+            // horizontal pixels per degree
+            mp->distance        = ((double) pn->width) / b;
+            break;
+        case _stereographic:
+            tpara = 1;
+            stereographic_erect(b/2.0, 0.0, &tx, &ty, & tpara);
+            mp->distance = pn->width/(2.0*tx);
+            break;
+        case _trans_mercator:
+            tpara = 1;
+            transmercator_erect(b/2.0, 0.0, &tx, &ty, & tpara);
+            mp->distance = pn->width/(2.0*tx);
+            break;
+        default:
+            // unknown
+            PrintError ("SetMakeParams: Unsupported panorama projection");
+            // no way to report an error back to the caller...
+            mp->distance = 1;
+        }
 
+        // calculate final scaling factor, that reverses the mp->distance
+        // scaling and applies the required output scaling factor
+        switch (im->format)
+        {
+        case _rectilinear:
+            // calculate distance for this projection
+            mp->scale[0] = (double) im->width / (2.0 * tan(a/2.0)) / mp->distance;
+            break;
+        case _equirectangular:
+        case _panorama:
+        case _fisheye_ff:
+        case _fisheye_circ:
+            mp->scale[0] = ((double) im->width) / a / mp->distance;
+            break;
+        default:
+            PrintError ("SetMakeParams: Unsupported input image projection");
+            // no way to report an error back to the caller...
+            mp->scale[1] = 1;
+        }
+        mp->scale[1]    = mp->scale[0];
+
+        /*
 	if(pn->format == _rectilinear)									// rectilinear panorama
 	{
 		mp->distance 	= (double) pn->width / (2.0 * tan(b/2.0));
@@ -834,6 +977,8 @@ void 	SetInvMakeParams( struct fDesc *stack, struct MakeParams *mp, Image *im ,
 			mp->scale[0] = ((double)pn->hfov / im->hfov) * ((double)im->width)/ ((double) pn->width); 
 		}
 	}
+        */
+
 	mp->shear[0] 	= -im->cP.shear_x / im->height;
 	mp->shear[1] 	= -im->cP.shear_y / im->width;
 	
@@ -919,6 +1064,28 @@ void 	SetInvMakeParams( struct fDesc *stack, struct MakeParams *mp, Image *im ,
 	{
 		SetDesc(stack[i],	sphere_tp_erect,		&(mp->distance)	); i++;	// Convert rectilinear to spherical
 	}
+   	else if(pn->format == _mercator)
+	{
+		SetDesc(stack[i],	mercator_erect,		&(mp->distance)	); i++;	// Convert sphere to sphere
+    }
+   	else if(pn->format == _trans_mercator)
+	{
+		SetDesc(stack[i],	transmercator_erect,		&(mp->distance)	); i++;	// Convert sphere to transverse mercator
+    }
+	else if(pn->format == _stereographic)
+	{
+		SetDesc(stack[i],	stereographic_erect,		&(mp->distance)	); i++;	// Convert sphere to stereographic
+    }
+   	else if(pn->format == _sinusoidal)
+	{
+		SetDesc(stack[i],	sinusoidal_erect,		&(mp->distance)	); i++;	// Convert sphere to sinusoidal
+	}
+   	else if(pn->format == _equirectangular) 
+	{
+		// no conversion needed		
+    } else {
+		PrintError("Projection type %d not supported, using equirectangular", pn->format);
+	}
 	
 	stack[i].func = (trfn)NULL;
 }
@@ -1128,7 +1295,7 @@ double distSphere( int num ){
 	Image sph;
 	int n[2];
 	struct 	MakeParams	mp;
-	struct  fDesc 		stack[15];
+	struct  fDesc        	stack[15];
 	CoordInfo b[2];
 	CoordInfo cp;
 	double lat[2], lon[2];  // latitude & longitude
@@ -1162,7 +1329,7 @@ double distSphere( int num ){
 		w2	= (double)g->im[ n[j] ].width  / 2.0 - 0.5;
 		
 		
-		execute_stack( 	(double)g->cpt[num].x[j] - w2,		// cartesian x-coordinate src
+		execute_stack_new( 	(double)g->cpt[num].x[j] - w2,		// cartesian x-coordinate src
 						(double)g->cpt[num].y[j] - h2,		// cartesian y-coordinate src
 						&x, &y, stack);
 
@@ -1215,7 +1382,7 @@ double distSphere( int num ){
 
 void pt_getXY(int n, double x, double y, double *X, double *Y){
 	struct 	MakeParams	mp;
-	struct  fDesc 		stack[15];
+	struct  fDesc        	stack[15];
 	double h2,w2;
 
 	SetInvMakeParams( stack, &mp, &g->im[ n ], &g->pano, 0 );
@@ -1223,7 +1390,7 @@ void pt_getXY(int n, double x, double y, double *X, double *Y){
 	w2	= (double)g->im[ n ].width  / 2.0 - 0.5;
 
 
-	execute_stack( 	x - w2,	y - h2,	X, Y, stack);
+	execute_stack_new( 	x - w2,	y - h2,	X, Y, stack);
 }
 
 // Return distance of points from a line
@@ -1289,7 +1456,7 @@ double rectDistSquared( int num )
 	double result;
 
 	struct 	MakeParams	mp;
-	struct  fDesc 		stack[15];
+	struct  fDesc        	stack[15];
 
 	
 
@@ -1306,7 +1473,7 @@ double rectDistSquared( int num )
 		w2	= (double)g->im[ n[j] ].width  / 2.0 - 0.5;
 		
 
-		execute_stack( 	(double)g->cpt[num].x[j] - w2,		// cartesian x-coordinate src
+		execute_stack_new( 	(double)g->cpt[num].x[j] - w2,		// cartesian x-coordinate src
 						(double)g->cpt[num].y[j] - h2,		// cartesian y-coordinate src
 						&x[j], &y[j], stack);
 		// test to check if inverse works
@@ -1316,7 +1483,7 @@ double rectDistSquared( int num )
 			struct 	MakeParams	mtest;
 			struct  fDesc 		stacktest[15];
 			SetMakeParams( stacktest, &mtest, &g->im[ n[j] ], &g->pano, 0 );
-			execute_stack( 	x[j],		// cartesian x-coordinate src
+			execute_stack_new( 	x[j],		// cartesian x-coordinate src
 							y[j],		// cartesian y-coordinate src
 						&xt, &yt, stacktest);
 			
@@ -2375,7 +2542,7 @@ void GetControlPointCoordinates(int i, double *x, double *y, AlignInfo *gl )
 	int j, n[2];
 
 	struct 	MakeParams	mp;
-	struct  fDesc 		stack[15];
+	struct  fDesc        	stack[15];
 
 	
 
@@ -2392,7 +2559,7 @@ void GetControlPointCoordinates(int i, double *x, double *y, AlignInfo *gl )
 		w2	= (double)gl->im[ n[j] ].width  / 2.0 - 0.5;
 		
 
-		execute_stack( 	(double)gl->cpt[i].x[j] - w2,		// cartesian x-coordinate src
+		execute_stack_new( 	(double)gl->cpt[i].x[j] - w2,		// cartesian x-coordinate src
 						(double)gl->cpt[i].y[j] - h2,		// cartesian y-coordinate src
 						&x[j], &y[j], stack);
 
