@@ -221,7 +221,12 @@ enum{				// Enumerates for Image.format
 	_spherical_tp	= 6,
 	_mirror			= 7,
 	_orthographic 	= 8,
-	_cubic			= 9
+	_cubic			= 9,
+	_stereographic  = 10,
+	_mercator       = 11,
+	_trans_mercator = 12,
+	_trans_panorama = 13,
+	_sinusoidal     = 14
 	};
 
 // A large rectangle
@@ -233,6 +238,14 @@ typedef struct{
 	pt_int32	right;
 	}	PTRect;
 
+typedef struct {
+	pt_int32		full_width;
+	pt_int32		full_height;
+	pt_int32		cropped_width;
+	pt_int32		cropped_height;
+	pt_int32		x_offset;
+	pt_int32		y_offset;
+} CropInfo;
 
 
 struct Image{
@@ -257,6 +270,8 @@ struct Image{
 typedef struct Image Image;
 
 
+
+
 struct TrformStr 				// This structure holds all image information
 	{
 		Image *src;				// Source image, must be supplied on entry
@@ -294,6 +309,7 @@ typedef struct TrformStr TrformStr;
 
 //void filter_main();
 
+#include "PTcommon.h"
 
 #endif // PANORAMA_H
 
