@@ -31,6 +31,7 @@ void RGBAtoARGB(UCHAR* buf, int width, int bitsPerPixel);
 void ARGBtoRGBA(UCHAR* buf, int width, int bitsPerPixel);
 int	readtif(Image *im, TIFF* tif);
 
+
 // image is allocated, but not image data
 
 int readtif(Image *im, TIFF* tif){
@@ -136,7 +137,11 @@ int readTIFF(Image *im, fullPath *sfile){
 		return -1;
 	}
 	result = readtif(im, tif);
-    	TIFFClose(tif);
+	
+	//Store name of TIFF file
+	strncpy(im->name, filename, 255);
+	
+	TIFFClose(tif);
 	return result;
 }
 
@@ -245,7 +250,7 @@ int writeTIFF(Image *im, fullPath *sfile){
 	}
 	TIFFSetField(tif, TIFFTAG_COMPRESSION, COMPRESSION_PACKBITS );	
 	TIFFSetField(tif, TIFFTAG_ORIENTATION, ORIENTATION_TOPLEFT );
-	TIFFSetField(tif, TIFFTAG_ROWSPERSTRIP, im->height );
+	TIFFSetField(tif, TIFFTAG_ROWSPERSTRIP, 1 );
 	
 	bufsize = TIFFScanlineSize(tif);
 	if(bufsize < im->bytesPerLine) bufsize = im->bytesPerLine;
